import csv
import re
import pandas as pd
import unicodedata

def load_unicode_ranges(unicode_csv):
    """Charge le fichier des plages Unicode en un dictionnaire."""
    df_ranges = pd.read_csv(unicode_csv)
    unicode_dict = {}

    for _, row in df_ranges.iterrows():
        unicode_dict[row["langue"]] = (row["debut"], row["fin"])

    return unicode_dict

def determine_alphabet(text, unicode_dict):
    """Détermine l'alphabet le plus fréquent dans un texte donné."""
    char_counts = {lang: 0 for lang in unicode_dict}

    for char in text:
        char_code = ord(char)
        for lang, (start, end) in unicode_dict.items():
            if start <= char_code <= end:
                char_counts[lang] += 1

    return max(char_counts, key=char_counts.get) if char_counts else None

def analyze_texts(text_csv, unicode_csv, output_csv):
    """Analyse les textes et assigne l'alphabet majoritaire pour chaque langue."""
    df_texts = pd.read_csv(text_csv)
    unicode_dict = load_unicode_ranges(unicode_csv)

    grouped_texts = df_texts.groupby("Label")["Text"].apply(lambda x: ' '.join(x)).reset_index()
    
    results = []
    
    for _, row in grouped_texts.iterrows():
        text = row["Text"]
        lang = row["Label"]
        alphabet = determine_alphabet(text, unicode_dict)
        results.append({"lang": lang, "alphabet_majoritaire": alphabet})

    df_results = pd.DataFrame(results)
    df_results.to_csv(output_csv, index=False)

def get_languages_by_group(csv_path):
    """
    Construit un dictionnaire : 
        group (str) -> liste de labels (langues) appartenant à ce groupe.
    """
    df = pd.read_csv(csv_path).dropna(subset=["lang", "alphabet_majoritaire"])
    languages_by_group = df.groupby("alphabet_majoritaire")["lang"].unique().to_dict()
    languages_by_group = {group: list(langs) for group, langs in languages_by_group.items()}
    return languages_by_group

if __name__ == "__main__":
    analyze_texts('anlp/anlp-kaggle/data/train_submission_clean.csv', 'anlp/anlp-kaggle/data/unicode_ranges.csv', 'anlp/anlp-kaggle/data/alphabets.csv')
