from sklearn.metrics import accuracy_score, classification_report, confusion_matrix


import numpy as np 
import pandas as pd 

from alphabet_analysis import get_languages_by_group


def tfdidf_eval(X_val, y_val, model):
    val_preds = model.predict(X_val)
    val_acc = accuracy_score(y_val, val_preds)
    print("Validation Accuracy:", val_acc)
    return accuracy_score

# ----------------------------------------
# Model Evaluation - Global and by Alphabet
# ----------------------------------------
def evaluate_model(model, X, y, dataset_name="Validation"):
    y_pred = model.predict(X)
    group_mapping = get_group_mapping("output_analysis.csv")

    groups_true = [group_mapping[label] for label in y]
    groups_pred = [group_mapping[label] for label in y_pred]

    print("Accuracy:", accuracy_score(y, y_pred))
    print("Confusion Matrix:\n", confusion_matrix(y, y_pred))
    print("Classification Report:\n", classification_report(y, y_pred))

    print("Accuracy for alphabets:", accuracy_score(groups_true, groups_pred))
    print("Confusion Matrix for alphabets:\n", confusion_matrix(groups_true, groups_pred))
    print("Classification Report for alphabets:\n", classification_report(groups_true, groups_pred))


def evaluate_model_by_group(model, X, y, dataset_name="Validation"):
    y_pred = model.predict(X)

    group_mapping = get_group_mapping("data/alphabets.csv")
    languages_by_group = get_languages_by_group('data/alphabets.csv')

    groups_true = np.array([group_mapping[label] for label in y])
    groups_pred = np.array([group_mapping[label] for label in y_pred])

    results = []
    for group in np.unique(groups_true):
        idx = groups_true == group
        acc = accuracy_score(y[idx], y_pred[idx])
        results.append({"group": group, "accuracy": acc})

    pd.DataFrame(results).to_csv(f"group_alphabet_results_{dataset_name}.csv", index=False)


# ----------------------------------------
# Helper - Get Alphabet Group Mapping
# ----------------------------------------
def get_group_mapping(csv_path):
    df = pd.read_csv(csv_path).dropna(subset=["lang", "alphabet_majoritaire"])
    df["lang"] = df["lang"].astype("category")

    df_mapping = df.drop_duplicates(subset=["lang"])
    categories = df["lang"].cat.categories

    mapping = {}
    for code, label in enumerate(categories):
        group = df_mapping[df_mapping["lang"] == label]["alphabet_majoritaire"].iloc[0]
        mapping[code] = group

    return mapping
