import torch
import torch.nn as nn
import xgboost as xgb
import torch.nn.functional as F

from sklearn.linear_model import LogisticRegression

# logistic regression model
clf = LogisticRegression(
    solver='saga',
    multi_class='multinomial',
    n_jobs=12,
    max_iter=400,
    verbose=1,
    tol=1e-3
)


# transformer encoder classifier model
class TransformerClassifier(nn.Module):
    def __init__(self, vocab_size, embedding_dim, num_heads, hidden_dim, num_layers, num_classes):
        super(TransformerClassifier, self).__init__()
        self.embedding = nn.Embedding(vocab_size, embedding_dim)
        self.position_encoding = nn.Parameter(torch.zeros(1, 500, embedding_dim))
        encoder_layer = nn.TransformerEncoderLayer(d_model=embedding_dim, nhead=num_heads, dim_feedforward=hidden_dim)
        self.transformer_encoder = nn.TransformerEncoder(encoder_layer, num_layers=num_layers)
        self.classifier = nn.Linear(embedding_dim, num_classes)

    def forward(self, x):
        x = self.embedding(x) + self.position_encoding[:, :x.size(1), :]
        x = self.transformer_encoder(x)
        x = self.classifier(x.mean(dim=1))
        return x
    

# xgb regressor model
xgb_model = xgb.XGBClassifier(n_estimators = 200,
                  learning_rate =0.01,
                  max_depth =5,
                  subsample =0.5,
                  colsample_bytree= 0.5, verbose = True, 
                  eval_metric='mlogloss')


# simple classifier dense NN
class SimpleClassifier(nn.Module):
    def __init__(self, input_dim, hidden_dim, num_classes):
        super(SimpleClassifier, self).__init__()
        self.fc1 = nn.Linear(input_dim, hidden_dim)
        self.fc2 = nn.Linear(hidden_dim, hidden_dim)
        self.fc3 = nn.Linear(hidden_dim, num_classes)
        self.dropout = nn.Dropout(0.3)

    def forward(self, x):
        x = F.relu(self.fc1(x))
        x = self.dropout(x)
        x = F.relu(self.fc2(x))
        x = self.dropout(x)
    
        x = self.fc3(x)
        x = F.softmax(x)
        return x

