from tqdm import tqdm 
import torch
from torch.utils.data import DataLoader
from torch.optim.lr_scheduler import ReduceLROnPlateau

import torch.optim as optim
import torch.nn.functional as F


def train_network(network, X_train, y_train, X_val, y_val):

    # Create data loaders
    train_dataset = torch.utils.data.TensorDataset(X_train, y_train)
    train_loader = DataLoader(train_dataset, batch_size=64, shuffle=True)

    val_dataset = torch.utils.data.TensorDataset(X_val, y_val)
    val_loader = DataLoader(val_dataset, batch_size=64, shuffle=False)

    # Define the optimizer
    optimizer = optim.Adam(network.parameters(), lr=0.005)



    # Set device to MPS if available
    device = torch.device("mps" if torch.backends.mps.is_available() else "cpu")
    network.to(device)

    # Define the optimizer and scheduler
    optimizer = optim.Adam(network.parameters(), lr=0.001)
    scheduler = ReduceLROnPlateau(optimizer, mode='min', factor=0.2, patience=3, verbose=True)

    # Training loop
    best_val_accuracy = -1000
    early_stopping_counter = 0
    for epoch in range(200):
        network.train()
        total_loss = 0
        for inputs, labels in train_loader:
            inputs, labels = inputs.to(device), labels.to(device)
            optimizer.zero_grad()
            outputs = network(inputs)
            loss = F.cross_entropy(outputs, labels)
            loss.backward()
            optimizer.step()
            total_loss += loss.item()

        print(f'Epoch {epoch+1}, Loss: {total_loss / len(train_loader)}')

        # Validation loop
        network.eval()
        total_val_loss = 0
        correct = 0
        with torch.no_grad():
            for inputs, labels in val_loader:
                inputs, labels = inputs.to(device), labels.to(device)
                outputs = network(inputs)
                loss = F.cross_entropy(outputs, labels)
                total_val_loss += loss.item()
                _, predicted = torch.max(outputs.data, 1)
                correct += (predicted == labels).sum().item()

        val_loss = total_val_loss / len(val_loader)
        val_accuracy = 100 * correct / len(val_dataset)
        print(f'Validation Loss: {val_loss}, Accuracy: {val_accuracy}%')
        if val_accuracy > best_val_accuracy:
            best_val_accuracy = val_accuracy
            torch.save(network.state_dict(), 'model.pth')
            early_stopping_counter = 0
        else:
            early_stopping_counter += 1

        if early_stopping_counter >= 10:  # Early stopping condition
            print("Early stopping triggered. No improvement for 5 consecutive epochs.")
            break

        scheduler.step(val_loss)

